package org.xbib.j2html.codegen.test.wattsi;

import org.xbib.javapoet.ClassName;
import org.xbib.javapoet.MethodSpec;
import org.xbib.javapoet.TypeSpec;
import java.io.IOException;
import java.nio.file.Path;
import java.nio.file.Paths;
import java.util.List;
import javax.lang.model.element.Modifier;
import org.jsoup.Jsoup;
import org.jsoup.nodes.Document;

public class WattsiGenerator {

    public static void main(String... args) throws IOException {
        Path source = Paths.get(args[0]);
        Document doc = Jsoup.parse(source.toFile(), "UTF-8", "https://html.spec.whatwg.org/");
        WattsiSource wattsi = new WattsiSource(doc);
        List<ElementDefinition> elements = wattsi.elementDefinitions();
        List<AttributeDefinition> attributes = wattsi.attributeDefinitions();
        for (ElementDefinition element : elements) {
            ClassName className = ClassName.get(
                    "org.xbib.j2html",
                    capitalize(element.name()) + "Tag"
            );
            TypeSpec.Builder type = TypeSpec.classBuilder(className)
                    .addModifiers(Modifier.PUBLIC);

            if (element.isObsolete()) {
                type.addAnnotation(Deprecated.class);
            }

            for (AttributeDefinition attribute : attributes) {
                if (attribute.appliesTo(element)) {
                    String name = methodName("with", attribute.name().split("-"));
                    MethodSpec.Builder setter = MethodSpec.methodBuilder(name)
                            .addModifiers(Modifier.PUBLIC)
                            .returns(className)
                            .addStatement("return this");

                    if (attribute.isObsolete()) {
                        setter.addAnnotation(Deprecated.class);
                    }

                    type.addMethod(setter.build());
                }
            }
            System.out.println(type.build());
        }
    }

    private static String methodName(String prefix, String... words) {
        String[] tmp = new String[words.length + 1];
        tmp[0] = prefix;
        System.arraycopy(words, 0, tmp, 1, words.length);
        return methodName(tmp);
    }

    private static String methodName(String... words) {
        String[] camelCase = new String[words.length];
        camelCase[0] = words[0];
        for (int i = 1; i < words.length; i++) {
            camelCase[i] = capitalize(words[i]);
        }
        return String.join("", camelCase);
    }

    private static String capitalize(String word) {
        return word.substring(0, 1).toUpperCase() + word.substring(1).toLowerCase();
    }
}
