package org.xbib.j2html.test.tags;

import java.io.InputStream;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;
import static org.xbib.j2html.TagCreator.fileAsEscapedString;
import static org.xbib.j2html.TagCreator.fileAsString;
import static org.xbib.j2html.TagCreator.scriptWithInlineFile_min;
import static org.xbib.j2html.TagCreator.styleWithInlineFile_min;
import static org.hamcrest.MatcherAssert.assertThat;
import static org.hamcrest.Matchers.is;

public class InlineStaticResourceTest {

    private static final String EOL = System.lineSeparator();

    @Test
    public void testAllTags() throws Exception {

        String expectedCss = "<style>body{background:#daa520;margin-bottom:10px;margin-left:10px;margin-right:10px;margin-top:10px}</style>";
        String expectedJs = "<script>(function(){var test=5;var tast=10;var testTast=test+tast;console.log(testTast);})();</script>";
        String expectedHtml = "<body>" + EOL + "    Any content" + EOL + "</body>" + EOL;
        String expectedEscapedHtml = "&lt;body&gt;" + EOL + "    Any content" + EOL + "&lt;/body&gt;" + EOL;
        String expectedAnyContent = "public class AnyContent {" + EOL + "}" + EOL;

        try (InputStream inputStream = getClass().getResourceAsStream("test.css")) {
            assertThat(styleWithInlineFile_min(inputStream).render(), is(expectedCss));
        }
        try (InputStream inputStream = getClass().getResourceAsStream("test-without-trailing-semis.css")) {
            assertThat(styleWithInlineFile_min(inputStream).render(), is(expectedCss));
        }
        try (InputStream inputStream = getClass().getResourceAsStream("test.js")) {
            assertThat(scriptWithInlineFile_min(inputStream).render(), is(expectedJs));
        }
        try (InputStream inputStream = getClass().getResourceAsStream("test.html")) {
            assertThat(fileAsString(inputStream).render(), is(expectedHtml));
        }
        try (InputStream inputStream = getClass().getResourceAsStream("test.html")) {
            assertThat(fileAsEscapedString(inputStream).render(), is(expectedEscapedHtml));
        }
        try (InputStream inputStream = getClass().getResourceAsStream("AnyContent.java")) {
            assertThat(fileAsString(inputStream).render(), is(expectedAnyContent));
        }
    }

    @Test
    public void testExceptionForBadPath() {
        Assertions.assertThrows(RuntimeException.class, () -> {
            styleWithInlineFile_min(null);
        });
    }
}
